﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Windows.Data;
using Stimulsoft.Report;
using Stimulsoft.Report.Components;
using ReportLargeAmountOfData.ViewModel;

namespace Gamanet.C4.Client.Converters
{
    /// <summary>
    /// Converts from a business report object to a stimulsoft report object
    /// </summary>
    [ValueConversion(typeof(IReport), typeof(StiReport))]
    public class BusinessReport2StiReportConverter : IValueConverter
    {
        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value produced by the binding source.</param>
        /// <param name="targetType">The type of the binding target property.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            var businessReport = value as IReport;
            if (businessReport == null)
                return null;

            var report = new StiReport();
            report.Load(businessReport.ReportUri);
            
            var removedDataSources = new List<string>();
            foreach (var bo in businessReport.BussinessObjects)
            {
                var dataSourceName = bo.Key;
                if (report.Dictionary.DataSources.Contains(dataSourceName))
                {
                    report.Dictionary.DataSources.Remove(report.Dictionary.DataSources[dataSourceName]);
                    report.RegBusinessObject(dataSourceName, bo.Value);
                    removedDataSources.Add(dataSourceName);
                }
                else
                {
                    report.RegBusinessObject(bo.Key, bo.Value);
                }
            }

            if (removedDataSources.Any())
            {
                report.Dictionary.SynchronizeBusinessObjects(1);

                foreach (var band in report
                    .GetComponents()
                    .OfType<StiDataBand>()
                    .Where(b => removedDataSources.Contains(b.DataSourceName)))
                {
                    var name = band.DataSourceName;
                    band.DataSourceName = null;
                    band.BusinessObjectGuid = report.Dictionary.BusinessObjects[name].Guid;
                }
            }

            try
            {
                report.Compile();

                foreach (var bo in businessReport.Parameters)
                    report[bo.Key] = bo.Value;
                
                return report;
            }
            catch (Exception xp)
            {
            }

            return null;
        }

        /// <summary>
        /// Converts a value. 
        /// </summary>
        /// <returns>
        /// A converted value. If the method returns null, the valid null value is used.
        /// </returns>
        /// <param name="value">The value that is produced by the binding target.</param>
        /// <param name="targetType">The type to convert to.</param>
        /// <param name="parameter">The converter parameter to use.</param>
        /// <param name="culture">The culture to use in the converter.</param>
        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotSupportedException();
        }
    }
}
